#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.16.1
 Author:         Kanashius

 Script Function:
	Make error handling easier by just calling _errCheck.

#ce ----------------------------------------------------------------------------
#include-once
Global $_Err_Callback = Default

; #FUNCTION# =======================================================================================
; Name ..........: _Err_Check
; Description ...: Checks if the @error is set (function call before this returned an error) and writes the error to the console.
; Syntax ........: _Err_Check($sMsg = Default, $bFatal = False, $return = -1, $iError = @error, $iExtended = @extended, $sExitCallback = "_exit", $iLine = @ScriptLineNumber)
; Parameters ....: $sMsg                  - The message to show in case of an error
;                  $bFatal                - If True, the $sExitCallback will be called to exit the application gracefully.
;                  $return                - The value to be returned by this function
;                  $iError                - The @error code
;                  $iExtended             - The @extended code
;                  $sExitCallback         - The function to call on exit
;                  $iLine                 - The ScriptLineNumber, where the error occurred
; Return values .: The value specified as $return and it will throw the same @error and @extended codes, the function was called with.
; Author ........: Kanashius
; ==================================================================================================
Func _Err_Check($sMsg = Default, $bFatal = False, $return = -1, $iError = @error, $iExtended = @extended, $sExitCallback = "_exit", $iLine = @ScriptLineNumber)
	$sMsg = $sMsg<>Default ? " >> "&$sMsg:""
	Local $sExtended = $iExtended<>0 ? ":"&$iExtended:""
	If $iError<>0 Then
		Local $iConsoleMessage = "Error (line "&$iLine&"): "&$iError&$sExtended&$sMsg
		ConsoleWrite($iConsoleMessage&@crlf)
		If $_Err_Callback<>Default Then
			Call(FuncName($_Err_Callback), $iConsoleMessage, $sMsg, $bFatal, $iError, $iExtended, $iLine)
			If @error Then
				_Err_Check("Error using the ""Call"" function, while handling another error: "&@crlf&@Tab&"Error (line "&$iLine&"): "&$iError&$sExtended&$sMsg&@crlf& _
						  "Check if the function """&$sExitCallback&""" (used for $bFatal errors) exists and has no parameters!"&@crlf)
				Exit
			EndIf
		EndIf
		If $bFatal Then
			Call($sExitCallback)
			If @error Then
				_Err_Check("Error using the ""Call"" function, while handling another error: "&@crlf&@Tab&"Error (line "&$iLine&"): "&$iError&$sExtended&$sMsg&@crlf& _
						  "Check if the function """&$sExitCallback&""" (used for $bFatal errors) exists and has no parameters!"&@crlf)
				Exit
			EndIf
		EndIf
	EndIf
	Return SetError(@error, @extended, $return)
EndFunc


; #FUNCTION# =======================================================================================
; Name ..........: _Err_SetCallback
; Description ...: Set a callback function, which is called for every error.
;                  The function must have parameters for:
; Syntax ........: _errCheck($sMsg = Default, $bFatal = False, $return = -1, $iError = @error, $iExtended = @extended, $sExitCallback = "_exit", $iLine = @ScriptLineNumber)
; Parameters ....: $sMsg                  - The message to show in case of an error
;                  $bFatal                - If True, the $sExitCallback will be called to exit the application gracefully.
;                  $return                - The value to be returned by this function
;                  $iError                - The @error code
;                  $iExtended             - The @extended code
;                  $sExitCallback         - The function to call on exit
;                  $iLine                 - The ScriptLineNumber, where the error occurred
; Return values .: The value specified as $return and it will throw the same @error and @extended codes, the function was called with.
; Author ........: Kanashius
; ==================================================================================================
Func _Err_SetCallback($sFunc = Default)
	If Not ($sFunc=Default or IsFunc($sFunc)) Then return SetError(1, 0, False)
	$_Err_Callback = $sFunc
	return True
EndFunc