#include <WindowsConstants.au3>
#include <WinAPI.au3>
#include <GDIPlus.au3>
#include <Math.au3>
#include "qrcode\QRCode.au3"
#include "ErrorHandling.au3"

Global $iDefaultPWLength = 60, $mMainGui[], $mConfig[], $sPathData = "data\", $sPathConfig = $sPathData&"config.ini"

_Err_SetCallback(_errorCallback)

_loadConfig()
If Not FileExists($sPathConfig) Then _saveConfig(True)

_QRCode_StartUp("qrcode\")
_Err_Check("QRCode startup failed", True)

_openGui()


Func _loadConfig()
	Local $mGeneral[], $sGeneral = "general"
	$mConfig[$sGeneral] = $mGeneral
	$mConfig[$sGeneral]["savePath"] = "out"

	Local $mGui[], $sGui = "gui"
	$mConfig[$sGui] = $mGui
	$mConfig[$sGui]["width"] = 800
	$mConfig[$sGui]["height"] = 600
	$mConfig[$sGui]["spacing"] = 5
	$mConfig[$sGui]["controlHeight"] = 25
	$mConfig[$sGui]["fontSize"] = 10
	$mConfig[$sGui]["fontWeight"] = 400
	$mConfig[$sGui]["fontAttribute"] = 0
	$mConfig[$sGui]["fontName"] = "Segoe Ui variable"

	Local $mQRCode[], $sQRCode = "qrCode"
	$mConfig[$sQRCode] = $mQRCode
	$mConfig[$sQRCode]["pixelSize"] = 4
	$mConfig[$sQRCode]["fontSize"] = 12
	$mConfig[$sQRCode]["spacing"] = 10
	$mConfig[$sQRCode]["font"] = "Consolas"

	For $sSection in MapKeys($mConfig)
		For $sKey in MapKeys($mConfig[$sSection])
			$mConfig[$sSection][$sKey] = IniRead($sPathConfig, $sSection, $sKey, $mConfig[$sSection][$sKey])
			$mConfig[$sSection][$sKey&"---original"] = IniRead($sPathConfig, $sSection, $sKey, $mConfig[$sSection][$sKey])
		Next
	Next
EndFunc

Func _saveConfig($bForceWrite = False)
	For $sSection in MapKeys($mConfig)
		For $sKey in MapKeys($mConfig[$sSection])
			If Not StringInStr($sKey, "---original") Then
				If $bForceWrite or $mConfig[$sSection][$sKey]<>$mConfig[$sSection][$sKey&"---original"] Then
					IniWrite($sPathConfig, $sSection, $sKey, $mConfig[$sSection][$sKey])
				EndIf
			EndIf
		Next
	Next
EndFunc

Func _openGui()
	; _GDIPlus_Startup()
	; _Err_Check("_GDIPlus_Startup failed", True)
	$mMainGui["width"] = $mConfig.gui.width
	$mMainGui["height"] = $mConfig.gui.height
	$mMainGui["bitmapQR"] = 0
	$mMainGui["updateQRCode"] = True
	Local $iControlHeight = $mConfig.gui.controlHeight, $iLeftRowWidth = 60, $iSpace = $mConfig.gui.spacing
	$mMainGui["path"] = ""
	$mMainGui["hGui"] = GUICreate("Wifi QRCode Generator", $mMainGui.width, $mMainGui.height)
	GUISetFont($mConfig.gui.fontSize, $mConfig.gui.fontWeight, $mConfig.gui.fontAttribute, $mConfig.gui.fontName, $mMainGui.hGui)

	Local $iTop = $iSpace, $iLeft = $iSpace
	$mMainGui["labelSSID"] = GUICtrlCreateLabel("SSID: ", $iLeft, $iTop+5, $iLeftRowWidth, $iControlHeight)
	$iLeft+=$iSpace+$iLeftRowWidth
	Local $iPixelWidth = 50, $iPixelLabelWidth = 100, $iSSIDWidth = $mMainGui.width-$iSpace*5-$iLeftRowWidth-$iPixelWidth-$iPixelLabelWidth
	$mMainGui["inputSSID"] = GUICtrlCreateInput("", $iLeft, $iTop, $iSSIDWidth, $iControlHeight)
	$iLeft += $iSSIDWidth+$iSpace
	GUICtrlCreateLabel("PixelSize: ", $iLeft, $iTop+5, $iPixelLabelWidth, $iControlHeight)
	$iLeft += $iPixelLabelWidth+$iSpace
	$mMainGui["inputPixelSize"] = GUICtrlCreateInput($mConfig.qrCode.pixelSize, $iLeft, $iTop, $iPixelWidth, $iControlHeight)
	$iTop += $iControlHeight+$iSpace
	$iLeft = $iSpace
	GUICtrlCreateLabel("Password: ", $iLeft, $iTop+3, $iLeftRowWidth, $iControlHeight)
	$iLeft += $iSpace+$iLeftRowWidth
	Local $iButtonGenerateWidth = 100, $iPWLengthFieldWidth = 50, $iInputPasswordWidth = $mMainGui.width-$iLeft-$iSpace*3-$iButtonGenerateWidth-$iPWLengthFieldWidth
	$mMainGui["inputPassword"] = GUICtrlCreateInput("", $iLeft, $iTop, $iInputPasswordWidth, $iControlHeight)
	$iLeft += $iInputPasswordWidth+$iSpace
	$mMainGui["buttonPassword"] = GUICtrlCreateButton("Generate", $iLeft, $iTop, $iButtonGenerateWidth, $iControlHeight)
	$iLeft += $iButtonGenerateWidth+$iSpace
	$mMainGui["inputPasswordLength"] = GUICtrlCreateInput("", $iLeft, $iTop, $iPWLengthFieldWidth, $iControlHeight)
	GUICtrlSetData($mMainGui.inputPasswordLength, $iDefaultPWLength)
	$iTop += $iControlHeight+$iSpace
	$iLeft = $iSpace
	Local $iButtonPointsWidth = 30, $iButtonSaveWidth = 50, $iPathWidth = $mMainGui.width-$iSpace*4-$iButtonPointsWidth-$iButtonSaveWidth
	$mMainGui["inputPath"] = GUICtrlCreateInput("", $iLeft, $iTop, $iPathWidth, $iControlHeight)
	$iLeft += $iPathWidth+$iSpace
	$mMainGui["buttonPath"] = GUICtrlCreateButton("...", $iLeft, $iTop, $iButtonPointsWidth, $iControlHeight)
	$iLeft += $iButtonPointsWidth+$iSpace
	$mMainGui["buttonSave"] = GUICtrlCreateButton("Save", $iLeft, $iTop, $iButtonSaveWidth, $iControlHeight)
	$iTop+=$iControlHeight+$iSpace

	_guiGeneratePassword()
	If @error=1 Then
		MsgBox(16, "Error", "Passwordlength must be between 1 and 63")
	Else
		_Err_Check("Password generation failed.")
	EndIf
	_selectFolder(@WorkingDir)
	_Err_Check("Folder selection failed.")

	GUIRegisterMsg($WM_COMMAND, "WM_COMMAND")
	GUISetState(@SW_SHOW, $mMainGui.hGui)


	$mMainGui["graphicGui"] = _GDIPlus_GraphicsCreateFromHWND($mMainGui.hGui)
	$mMainGui["graphicWidth"] = $mMainGui.width
	$mMainGui["graphicHeight"] = $mMainGui.height-$iTop
	$mMainGui["bitmapBG"] = _GDIPlus_BitmapCreateFromGraphics($mMainGui.graphicWidth, $mMainGui.graphicHeight, $mMainGui.graphicGui)
	$mMainGui["graphic"] = _GDIPlus_ImageGetGraphicsContext($mMainGui.bitmapBG)
	_GDIPlus_GraphicsClear($mMainGui.graphic, 0xFFFFFFFF)

	While 1
		Local $iMsg = GUIGetMsg()
		If $iMsg = -3 Then _exit()
		If $iMsg = $mMainGui.buttonPassword Then
			_guiGeneratePassword()
			If @error=1 Then
				MsgBox(16, "Error", "Passwordlength must be between 1 and 63")
			Else
				_Err_Check("Password generation failed.")
			EndIf
		EndIf
		If $iMsg = $mMainGui.buttonPath Then
			_selectFolder()
			_Err_Check("Folder selection failed.")
		EndIf
		If $mMainGui.updateQRCode or $iMsg = $mMainGui.buttonSave Then
			$mMainGui.updateQRCode = False
			_drawQRCode()
			If @error Then
				_Err_Check("Error drawing QRCode.")
			ElseIf $iMsg = $mMainGui.buttonSave Then
				_updatePath()
				_Err_Check("Error updating path.")
				_GDIPlus_ImageSaveToFile($mMainGui.bitmapQR, GUICtrlRead($mMainGui.inputPath))
				_Err_Check("Error saving QRCode image.")
				Local $sPath = $mMainGui.path&"\data\"&GUICtrlRead($mMainGui.inputSSID)&".txt"
				$hFile = FileOpen($sPath, BitOr(2,8,128))
				_Err_Check("Error opening password file.")
				FileWrite($hFile, GUICtrlRead($mMainGui.inputPassword))
				_Err_Check("Error writing password file.")
				FileClose($hFile)
			EndIf
		EndIf
		_GDIPlus_GraphicsDrawImageRect($mMainGui.graphicGui, $mMainGui.bitmapBG, 0, $iTop, $mMainGui.graphicWidth, $mMainGui.graphicHeight)
	WEnd
EndFunc

Func _drawQRCode()
	_GDIPlus_BitmapDispose($mMainGui.bitmapQR)
	Local $iSpacing = $mConfig.qrCode.spacing, $sFont = $mConfig.qrCode.font, $iFontSize = $mConfig.qrCode.fontSize

	Local $iPixelSize = Int(GUICtrlRead($mMainGui.inputPixelSize))
	If $iPixelSize<1 Then return SetError(1, 0, False) ; QRCode Pixelsize needs to be larger then 0!
	Local $sSSID = GUICtrlRead($mMainGui.inputSSID)
	Local $sPassword = GUICtrlRead($mMainGui.inputPassword)
	Local $sQRCodeData = _generateWifiQRCodeData($sSSID, $sPassword)
	If @error Then return SetError(2, 0, False) ; WifiQRCodeData could not be generated
	Local $arQRData = _QRCode_GetQRCode($sQRCodeData)
	If @error Then return SetError(3, 0, False) ; QRCode could not be generated

	Local $hFormat = _GDIPlus_StringFormatCreate()
	Local $hFamily = _GDIPlus_FontFamilyCreate($sFont)
	Local $hFont = _GDIPlus_FontCreate($hFamily, $iFontSize, 1)
	Local $tLayout = _GDIPlus_RectFCreate(0, 0, $mMainGui.graphicWidth, $mMainGui.graphicHeight)

	Local $arSizeSSID = _getStringWidth("SSID: "&$sSSID, $mMainGui.graphicGui, $hFamily, $hFont, $hFormat, $tLayout)
	If @error Then return SetError(4, 1, False) ; Width of SSID could not be calculated
	Local $arSizePassword = _getStringWidth("Password: "&@crlf&$sPassword, $mMainGui.graphicGui, $hFamily, $hFont, $hFormat, $tLayout)
	If @error Then return SetError(4, 2, False) ; Width of Password could not be calculated
	Local $arSizePasswordLabel = _getStringWidth("Password:", $mMainGui.graphicGui, $hFamily, $hFont, $hFormat, $tLayout)
	If @error Then return SetError(4, 3, False) ; Width of Password label could not be calculated
	Local $iQRWidth = $iPixelSize*UBound($arQRData), $iQRHeight = $iPixelSize*UBound($arQRData, 2)
	Local $iImageWidth = _Max(_Max(IsArray($arSizeSSID)?$arSizeSSID[2]:0, IsArray($arSizePassword)?$arSizePassword[2]:0), $iQRWidth)
	Local $iImageHeight = (IsArray($arSizeSSID)?$arSizeSSID[3]:0)+(IsArray($arSizePassword)?$arSizePassword[3]:0)+$iQRHeight+$iSpacing*2

	$mMainGui["bitmapQR"] = _GDIPlus_BitmapCreateFromGraphics($iImageWidth, $iImageHeight, $mMainGui.graphicGui)
	Local $hContext = _GDIPlus_ImageGetGraphicsContext($mMainGui.bitmapQR)
	Local $brushBlack = _GDIPlus_BrushCreateSolid(0xFF000000)
	Local $brushWhite = _GDIPlus_BrushCreateSolid(0xFFFFFFFF)
	_GDIPlus_GraphicsClear($hContext, 0xFFFFFFFF)

	Local $iLeft = ($iImageWidth-$iQRWidth)/2
	_QRCode_DrawQRCode($hContext, $arQRData, $iPixelSize, $iLeft, 0, 0xFF000000, 0xFFFFFFFF)
	If @error Then _Err_Check("QRCode drawing failed.")
#cs
	_GDIPlus_GraphicsSetSmoothingMode($hContext, 0)
	For $x=0 to UBound($arQRData)-1 Step 1
		For $y=0 to UBound($arQRData, 2)-1 Step 1
			If $arQRData[$x][$y] Then
				_GDIPlus_GraphicsFillRect($hContext, $iLeft+$x*$iPixelSize, $y*$iPixelSize, $iPixelSize, $iPixelSize, $brushBlack)
			Else
				_GDIPlus_GraphicsFillRect($hContext, $iLeft+$x*$iPixelSize, $y*$iPixelSize, $iPixelSize, $iPixelSize, $brushWhite)
			EndIf
		Next
	Next
#ce
	If IsArray($arSizeSSID) Then
		$tLayout = _GDIPlus_RectFCreate($iImageWidth/2-$arSizeSSID[2]/2, $iQRHeight+$iSpacing, $arSizeSSID[2], $arSizeSSID[3])
		_GDIPlus_GraphicsDrawStringEx($hContext, "SSID: "&$sSSID, $hFont, $tLayout, $hFormat, $brushBlack)
	EndIf
	If IsArray($arSizePassword) Then
		$tLayout = _GDIPlus_RectFCreate($iImageWidth/2-$arSizePasswordLabel[2]/2, $iQRHeight+$arSizeSSID[3]+$iSpacing*2, $arSizePasswordLabel[2], $arSizePasswordLabel[3])
		_GDIPlus_GraphicsDrawStringEx($hContext, "Password:", $hFont, $tLayout, $hFormat, $brushBlack)

		$tLayout = _GDIPlus_RectFCreate($iImageWidth/2-$arSizePassword[2]/2, $iQRHeight+$arSizeSSID[3]+$arSizePasswordLabel[3]+$iSpacing*2, $arSizePassword[2], $arSizePassword[3])
		_GDIPlus_GraphicsDrawStringEx($hContext, $sPassword, $hFont, $tLayout, $hFormat, $brushBlack)
	EndIf
	_GDIPlus_GraphicsClear($mMainGui.graphic, 0xFFFFFFFF)
	_GDIPlus_GraphicsDrawImageRect($mMainGui.graphic, $mMainGui.bitmapQR, $mMainGui.graphicWidth/2-$iImageWidth/2, $mMainGui.graphicHeight/2-$iImageHeight/2, $iImageWidth, $iImageHeight)
	_GDIPlus_GraphicsDispose($hContext)
	_GDIPlus_BrushDispose($brushBlack)
	_GDIPlus_BrushDispose($brushWhite)
	_GDIPlus_FontDispose($hFont)
	_GDIPlus_FontFamilyDispose($hFamily)
	_GDIPlus_StringFormatDispose($hFormat)
	return True
EndFunc

Func _getStringWidth($sString, $hGraphic, $hFamily, $hFont, $hFormat, $tLayout)
	$aInfo = _GDIPlus_GraphicsMeasureString($hGraphic, $sString, $hFont, $tLayout, $hFormat)
	If @error Then return SetError(1, 0, -1) ; _GDIPlus_GraphicsMeasureString failed
	Local $arRes = [DllStructGetData($aInfo[0], "X"), DllStructGetData($aInfo[0], "Y"), DllStructGetData($aInfo[0], "Width"), DllStructGetData($aInfo[0], "Height"), $aInfo[1], $aInfo[2]]
	return $arRes
EndFunc

Func _exit()
	If MapExists($mMainGui, "bitmapBG") Then _GDIPlus_BitmapDispose($mMainGui.bitmapBG)
	If MapExists($mMainGui, "graphic") Then _GDIPlus_GraphicsDispose($mMainGui.graphic)
	If MapExists($mMainGui, "graphicGui") Then _GDIPlus_GraphicsDispose($mMainGui.graphicGui)
	; _GDIPlus_Shutdown()
	_QRCode_Shutdown()
	_Err_Check("QRCode shutdown error")
	Exit
EndFunc

Func WM_COMMAND($hWnd, $iMsg, $wParam, $lParam)
	Local $nNotifyCode = _WinAPI_HiWord($wParam)
	Local $iId = _WinAPI_LoWord($wParam)
	Local $hCtrl = $lParam
	If $nNotifyCode=1024 and ($iId=$mMainGui.inputSSID or $iId = $mMainGui.inputPassword or $iId = $mMainGui.inputPixelSize) Then
		_updatePath()
		If @error Then _Err_Check("Error updating the save file path")
		$mMainGui.updateQRCode = True
	EndIf
EndFunc

Func _selectFolder($sFolder = Default)
	If $sFolder=Default Then $sFolder = FileSelectFolder("Select Folder", @WorkingDir, BitOR(1,2,4), "", $mMainGui.hGui)
	If Not @error Then
		$mMainGui.path = $sFolder
		_updatePath()
		If @error Then return SetError(1, 0, -1) ; Error updating the save file path
	EndIf
EndFunc

; CONTINUE HERE

Func _updatePath()
	GUICtrlSetData($mMainGui.inputPath, $mMainGui.path&"\data\"&GUICtrlRead($mMainGui.inputSSID)&".jpg")
EndFunc

Func _guiGeneratePassword()
	Local $iPasswordLength = Int(GUICtrlRead($mMainGui.inputPasswordLength))
	If $iPasswordLength<=0 or $iPasswordLength>63 Then return SetError(1, 0, False) ; Passwordlength must be between 1 and 63
	Local $sPassword = _genRandomPassword($iPasswordLength)
	If @error Then return SetError(2, 0, False) ; Password generation failed
	GUICtrlSetData($mMainGui.inputPassword, $sPassword)
	; qrcode is updated through WM_COMMAND
	return True
EndFunc

Func _generateWifiData($sSSID)
	return _generateWifiQRCodeData($sSSID, _genRandomPassword($iDefaultPWLength))
EndFunc

Func _generateWifiQRCodeData($sSSID, $sPassword)
	Local $sData = "WIFI:S:"&$sSSID&";T:WPA;P:"&$sPassword&";H:false;"
	return $sData
EndFunc

Func _genRandomPassword($iLength, $iType = 0)
	Local $arChars = ["!", "#", "$", "%", "&", "(", ")", "*", "+", ",", "-", ".", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "<", "=", ">", "?", "@", "A", "B", "C", _
					  "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "]", "^", "_", "a", "b", "c", "d", "e", "f", _
					  "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "}", "~"]
	SRandom(Mod(TimerInit(), 100000000))
	Local $sPassword = ""
	If $iType = 0 Then
		For $i=0 to $iLength-1 Step 1
			$sPassword &= $arChars[Random(0, UBound($arChars)-1,1)]
		Next
	ElseIf $iType = 1 Then ; Everything except ; and : => Delimeters in QR-Codes for sharing
		For $i=0 to $iLength-1 Step 1
		Local $iNumber = Random(33,126,1)
		If $iNumber = 58 or $iNumber = 59 Then
			$i-=1
		Else
			$sPassword &= Chr($iNumber)
		EndIf
	Next
	ElseIf $iType = 2 Then ; Everything allowed
		For $i=0 to $iLength-1 Step 1
			$sPassword &= Chr(Random(33,126,1))
		Next
	EndIf
	return $sPassword
EndFunc

Func _errorCallback($iConsoleMessage, $sMsg, $bFatal, $iError, $iExtended, $iLine)
	MsgBox(16, ($bFatal?"Fatal ":"")&"Error", $iConsoleMessage)
EndFunc